On 10/20/22 19:52, Cui, Lili via Gcc-patches wrote:
Hi Honza,

Gentle ping  
https://gcc.gnu.org/pipermail/gcc-patches/2022-September/601934.html

gcc/ChangeLog

   * ipa-inline-analysis.cc (do_estimate_edge_time): Add function attribute
   judgement for INLINE_HINT_known_hot hint.

gcc/testsuite/ChangeLog:

   * gcc.dg/ipa/inlinehint-6.c: New test.
---
  gcc/ipa-inline-analysis.cc              | 13 ++++---
  gcc/testsuite/gcc.dg/ipa/inlinehint-6.c | 47 +++++++++++++++++++++++++
  2 files changed, 56 insertions(+), 4 deletions(-)
  create mode 100644 gcc/testsuite/gcc.dg/ipa/inlinehint-6.c

diff --git a/gcc/ipa-inline-analysis.cc b/gcc/ipa-inline-analysis.cc
index 1ca685d1b0e..7bd29c36590 100644
--- a/gcc/ipa-inline-analysis.cc
+++ b/gcc/ipa-inline-analysis.cc
@@ -48,6 +48,7 @@ along with GCC; see the file COPYING3.  If not see
  #include "ipa-utils.h"
  #include "cfgexpand.h"
  #include "gimplify.h"
+#include "attribs.h"
/* Cached node/edge growths. */
  fast_call_summary<edge_growth_cache_entry *, va_heap> *edge_growth_cache = 
NULL;
@@ -249,15 +250,19 @@ do_estimate_edge_time (struct cgraph_edge *edge, sreal 
*ret_nonspec_time)
        hints = estimates.hints;
      }
- /* When we have profile feedback, we can quite safely identify hot
-     edges and for those we disable size limits.  Don't do that when
-     probability that caller will call the callee is low however, since it
+  /* When we have profile feedback or function attribute, we can quite safely
+     identify hot edges and for those we disable size limits.  Don't do that
+     when probability that caller will call the callee is low however, since it
       may hurt optimization of the caller's hot path.  */
-  if (edge->count.ipa ().initialized_p () && edge->maybe_hot_p ()
+  if ((edge->count.ipa ().initialized_p () && edge->maybe_hot_p ()
        && (edge->count.ipa () * 2
          > (edge->caller->inlined_to
             ? edge->caller->inlined_to->count.ipa ()
             : edge->caller->count.ipa ())))
+      || (lookup_attribute ("hot", DECL_ATTRIBUTES (edge->caller->decl))
+         != NULL
+        && lookup_attribute ("hot", DECL_ATTRIBUTES (edge->callee->decl))
+         != NULL))
      hints |= INLINE_HINT_known_hot;

Is the theory here that if the user has marked the caller and callee as hot, then we're going to assume an edge between them is hot too?  That's not necessarily true, it could be they're both hot, but via other call chains.  But it's probably a reasonable heuristic in practice.


OK


jeff


Reply via email to